@@ -6,6 +6,7 @@
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
  * Martin Lambers <marlam@marlam.de>
  * Jay Soffian <jaysoffian@gmail.com> (Mac OS X keychain support)
+ * Satoru SATOH <satoru.satoh@gmail.com> (GNOME keyring support)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -44,21 +45,24 @@ extern int optind;
 # include <syslog.h>
 #endif
 #ifdef W32_NATIVE
-#include <io.h>
-#include <fcntl.h>
-#include <windows.h>
-#include <winsock2.h>
+# include <io.h>
+# include <fcntl.h>
+# include <windows.h>
+# include <winsock2.h>
 #elif defined DJGPP
-#include <io.h>
-#include <fcntl.h>
-#include <netdb.h>
-#include <arpa/inet.h>
+# include <io.h>
+# include <fcntl.h>
+# include <netdb.h>
+# include <arpa/inet.h>
 #else /* UNIX */
-#include <netdb.h>
-#include <arpa/inet.h>
+# include <netdb.h>
+# include <arpa/inet.h>
 #endif
-#ifdef HAVE_KEYCHAIN
-#include <Security/Security.h>
+#ifdef HAVE_GNOMEKEYRING
+# include <gnome-keyring.h>
+#endif
+#ifdef HAVE_MACOSXKEYRING
+# include <Security/Security.h>
 #endif
 
 #include "getpass.h"
@@ -80,15 +84,18 @@ extern int optind;
 #ifdef W32_NATIVE
 #define SYSCONFFILE	"msmtprc.txt"
 #define USERCONFFILE	"msmtprc.txt"
-#define NETRCFILE	"netrc.txt"
+#define SYSNETRCFILE	"netrc.txt"
+#define USERNETRCFILE	"netrc.txt"
 #elif defined (DJGPP)
 #define SYSCONFFILE	"msmtprc"
 #define USERCONFFILE	"_msmtprc"
-#define NETRCFILE	"_netrc"
+#define SYSNETRCFILE	"netrc"
+#define USERNETRCFILE	"_netrc"
 #else /* UNIX */
 #define SYSCONFFILE	"msmtprc"
 #define USERCONFFILE	".msmtprc"
-#define NETRCFILE	".netrc"
+#define SYSNETRCFILE	".netrc"
+#define USERNETRCFILE	".netrc"
 #endif
 
 /* The name of this program */
@@ -282,11 +289,16 @@ char *msmtp_sanitize_string(char *str)
 
 char *msmtp_password_callback(const char *hostname, const char *user)
 {
-    char *homedir;
+    char *netrc_directory;
     char *netrc_filename;
     netrc_entry *netrc_hostlist;
     netrc_entry *netrc_host;
-#ifdef HAVE_KEYCHAIN
+#ifdef HAVE_GNOMEKEYRING
+    const char *protocol = "smtp";
+    GList *found_list = NULL;
+    GnomeKeyringNetworkPasswordData *found;
+#endif
+#ifdef HAVE_MACOSXKEYRING
     void *password_data;
     UInt32 password_length;
     OSStatus status;
@@ -297,9 +309,9 @@ char *msmtp_password_callback(const char *hostname, const char *user)
     char *gpw;
     char *password = NULL;
 
-    homedir = get_homedir();
-    netrc_filename = get_filename(homedir, NETRCFILE);
-    free(homedir);
+    netrc_directory = get_homedir();
+    netrc_filename = get_filename(netrc_directory, USERNETRCFILE);
+    free(netrc_directory);
     if ((netrc_hostlist = parse_netrc(netrc_filename)))
     {
 	if ((netrc_host = search_netrc(netrc_hostlist, hostname, user)))
@@ -310,25 +322,66 @@ char *msmtp_password_callback(const char *hostname, const char *user)
     }
     free(netrc_filename);
 
-#ifdef HAVE_KEYCHAIN
-    if (SecKeychainFindInternetPassword(
-		NULL,
-		strlen(hostname), hostname,
-		0, NULL,
-		strlen(user), user,
-		0, (char *)NULL,
-		0,
-		kSecProtocolTypeSMTP,
-		kSecAuthenticationTypeDefault,
-		&password_length, &password_data,
-		NULL) == noErr)
+    if (!password)
     {
-	password = xmalloc((password_length + 1) * sizeof(char));
-	strncpy(password, password_data, (size_t)password_length);
-	password[password_length] = '\0';
-	SecKeychainItemFreeContent(NULL, password_data);
+	netrc_directory = get_sysconfdir();
+	netrc_filename = get_filename(netrc_directory, SYSNETRCFILE);
+	free(netrc_directory);
+	if ((netrc_hostlist = parse_netrc(netrc_filename)))
+	{
+	    if ((netrc_host = search_netrc(netrc_hostlist, hostname, user)))
+	    {
+		password = xstrdup(netrc_host->password);
+	    }
+	    free_netrc_entry_list(netrc_hostlist);
+	}
+	free(netrc_filename);
     }
-#endif /* HAVE_KEYCHAIN */
+
+#ifdef HAVE_GNOMEKEYRING
+    if (!password)
+    {
+    	g_set_application_name(PACKAGE);
+    	if (gnome_keyring_find_network_password_sync(
+		    user,     /* user */
+		    NULL,     /* domain */
+		    hostname, /* server */
+		    NULL,     /* object */
+		    protocol, /* protocol */
+		    NULL,     /* authtype */
+		    0,        /* port */
+		    &found_list) == GNOME_KEYRING_RESULT_OK)
+	{
+	    found = (GnomeKeyringNetworkPasswordData *) found_list->data;
+	    if (found->password)
+		password = g_strdup(found->password);
+	}
+	gnome_keyring_network_password_list_free(found_list);
+    }
+#endif /* HAVE_GNOMEKEYRING */
+
+#ifdef HAVE_MACOSXKEYRING
+    if (!password)
+    {
+	if (SecKeychainFindInternetPassword(
+	    	    NULL,
+	    	    strlen(hostname), hostname,
+	    	    0, NULL,
+	    	    strlen(user), user,
+	    	    0, (char *)NULL,
+	    	    0,
+	    	    kSecProtocolTypeSMTP,
+	    	    kSecAuthenticationTypeDefault,
+	    	    &password_length, &password_data,
+	    	    NULL) == noErr)
+	{
+	    password = xmalloc((password_length + 1) * sizeof(char));
+	    strncpy(password, password_data, (size_t)password_length);
+	    password[password_length] = '\0';
+	    SecKeychainItemFreeContent(NULL, password_data);
+	}
+    }
+#endif /* HAVE_MACOSXKEYRING */
 
     /* Do not let getpass() read from stdin, because we read the mail from 
      * there. DJGPP's getpass() always reads from stdin. On W32, gnulib's
@@ -348,7 +401,7 @@ char *msmtp_password_callback(const char *hostname, const char *user)
 	    getpass_uses_tty = 1;
 	    fclose(tty);
 	}
-# endif
+#endif
 	if (getpass_uses_tty)
 	{
 	    prompt = xasprintf(_("password for %s at %s: "), user, hostname);
@@ -360,7 +413,7 @@ char *msmtp_password_callback(const char *hostname, const char *user)
 	    }
 	}
     }
-    
+
     return password;
 }
 
@@ -2223,6 +2276,18 @@ void msmtp_print_version(void)
     printf(_(", LOCALEDIR is %s"), LOCALEDIR);
 #else
     printf(_("disabled"));
+#endif
+    printf("\n");
+    printf(_("Keyring support: "));
+#if !defined HAVE_GNOMEKEYRING && !defined HAVE_MACOSXKEYRING
+    printf(_("none"));
+#else
+# ifdef HAVE_GNOMEKEYRING
+    printf(_("Gnome "));
+# endif
+# ifdef HAVE_MACOSXKEYRING
+    printf(_("MacOS "));
+# endif
 #endif
     printf("\n");
     sysconfdir = get_sysconfdir();
