@@ -32,6 +32,7 @@
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
+#include <unistd.h>
 
 #ifdef HAVE_LIBGSASL
 # include <gsasl.h>
@@ -1503,7 +1504,7 @@ int smtp_send_envelope(smtp_server_t *srv,
 		    return e;
 		}
 		status = smtp_msg_status(msg);
-		if (status != 250)
+		if (status != 250 && status != 251)
 		{
 		    *error_msg = msg;
 		    *errstr = xasprintf(_("recipient address %s not "
@@ -1892,5 +1893,5 @@ void smtp_close(smtp_server_t *srv)
 	tls_close(&srv->tls);
     }
 #endif /* HAVE_TLS */
-    net_close_socket(srv->fd);
+    close(srv->fd);
 }
